package ink.wgink.module.activiti.service.oa.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.module.activiti.service.oa.IOaFormReportService;
import ink.wgink.module.form.service.report.IFormReportService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResultList;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: OaFormReportServiceImpl
 * @Description: oa表单上报
 * @Author: wanggeng
 * @Date: 2022/3/22 22:05
 * @Version: 1.0
 */
@Service
public class OaFormReportServiceImpl extends DefaultBaseService implements IOaFormReportService {

    @Autowired
    private IFormReportService formReportService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private TaskService taskService;

    @Override
    public void save(String processDefinitionId, String formCode, Integer formVersion, Map<String, Object> params) {
        String userId = securityComponent.getCurrentUser().getUserId();
        saveByUserId(userId, processDefinitionId, formCode, formVersion, params);
    }

    @Override
    public void saveByToken(String token, String processDefinitionId, String formCode, Integer formVersion, Map<String, Object> params) {
        String userId = getAppTokenUser(token).getId();
        saveByUserId(userId, processDefinitionId, formCode, formVersion, params);
    }

    @Override
    public void saveByUserId(String userId, String processDefinitionId, String formCode, Integer formVersion, Map<String, Object> params) {
        LOG.debug("保存表单");
        String uid = formReportService.saveAndReturnId(formCode, formVersion, params);
        LOG.debug("定义流程发起人");
        Map<String, Object> variables = getHashMap(2);
        variables.put(KEY_FORM_CODE, formCode);
        variables.put(KEY_FORM_VERSION, formVersion);
        variables.put(KEY_REPORT_UID, uid);
        variables.put(KEY_START_ASSIGNEE, userId);
        LOG.debug("发起流程");
        ProcessInstance processInstance = runtimeService.startProcessInstanceById(processDefinitionId, variables);
        LOG.debug("更新表单数据的流程实例ID");
        formReportService.updateProcessInstanceId(formCode, formVersion, uid, processInstance.getProcessInstanceId());
        LOG.debug("执行第一个节点");
        Task task = taskService.createTaskQuery().processInstanceId(processInstance.getId()).taskCandidateOrAssigned(userId).singleResult();
        if (task == null) {
            throw new SearchException("任务不存在");
        }
        taskService.complete(task.getId(), params, true);
    }

    @Override
    public void update(String taskId, String formCode, Integer formVersion, Integer isNeedClaim, String uid, Map<String, Object> params) {
        String userId = securityComponent.getCurrentUser().getUserId();
        updateByUserId(userId, taskId, formCode, formVersion, isNeedClaim, uid, params);
    }

    @Override
    public void updateByToken(String token, String taskId, String formCode, Integer formVersion, Integer isNeedClaim, String uid, Map<String, Object> params) {
        String userId = getAppTokenUser(token).getId();
        updateByUserId(userId, taskId, formCode, formVersion, isNeedClaim, uid, params);
    }

    @Override
    public void updateByUserId(String userId, String taskId, String formCode, Integer formVersion, Integer isNeedClaim, String uid, Map<String, Object> params) {
        formReportService.update(formCode, formVersion, uid, params);
        if (isNeedClaim == 1) {
            LOG.debug("认领任务");
            taskService.claim(taskId, userId);
        }
        taskService.complete(taskId, params, true);
    }

    @Override
    public Map<String, Object> get(String formCode, Integer formVersion, String uid) {
        return formReportService.get(formCode, formVersion, uid);
    }

    @Override
    public List<Map<String, Object>> list(String formCode, Integer formVersion, Map<String, Object> params) {
        return formReportService.list(formCode, formVersion, params);
    }

    @Override
    public SuccessResultList<List<Map<String, Object>>> listPage(String formCode, Integer formVersion, ListPage page) {
        return formReportService.listPage(formCode, formVersion, page);
    }
}
