function FormUtil(layui, viewer) {
    var $ = layui.$;
    var layer = layui.layer;
    var upload = layui.upload;
    var laydate = layui.laydate;
    var Viewer = viewer;
    var restAjax = layui.restajax;
    var viewerObj = {};

    /**
     * 初始化图片上传
     * @param fieldName
     * @param maxCount
     */
    this.initUploadImage = function (fieldName, maxCount) {
        maxCount = maxCount ? parseInt(maxCount) : 9
        maxCount = maxCount < 0 ? 9 : maxCount;
        var id = '#' + fieldName;
        var fileBoxId = id + 'FileBox';
        var uploadBtnId = id + 'UploadBtn';
        var deleteBtnClass = '.delete-' + fieldName + '-btn';

        function init(callback) {
            var fileIds = $(id).val();
            var fileIdArray = fileIds ? fileIds.split(',') : [];

            var html = '';
            for (var i = 0, fileId; fileId = fileIdArray[i++];) {
                html += [
                    '<div class="form-upload-image">',
                    '   <img src="route/file/download/true/' + fileId + '" align="加载失败">',
                    '   <a class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-' + fieldName + '-btn" href="javascript:void(0);" data-id="' + fileId + '" data-name="' + fieldName + '">',
                    '       <i class="fa fa-trash-o"></i>',
                    '   </a>',
                    '</div>',
                ].join('');
            }

            $(fileBoxId).empty();
            $(fileBoxId).append(html);

            if (fileIdArray.length < maxCount) {
                $(uploadBtnId).removeClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', false);
            } else {
                $(uploadBtnId).addClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', true);
            }

            callback ? callback() : '';
        }

        function addClick() {
            upload.render({
                elem: uploadBtnId,
                url: 'api/file/v2/upload-image',
                accept: 'images',
                acceptMime: 'image/*',
                field: 'image',
                exts: 'jpg|png|gif|bmp|jpeg',
                name: fieldName,
                done: function (res, index, upload) {
                    var name = this.name;
                    var files = $('#' + this.name).val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId;
                    $('#' + name).val(files);
                    init(function () {
                        viewerObj[name].update();
                    });
                },
                error: function (index, upload) {
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                    var percent = n + '%'
                    console.log(percent);
                    console.log(elem);
                    console.log(res);
                    console.log(index);
                    element.progress('demo-' + index, n + '%');
                }
            });
            $(document).on('click', deleteBtnClass, function () {
                var name = this.dataset.name;
                var id = this.dataset.id;
                var files = $('#' + name).val().replace(id, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) == ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) == ',') {
                    files = files.substring(0, files.length - 1);
                }
                $('#' + name).val(files);
                init(function () {
                    viewerObj[name].update();
                });
            });
        }

        init(function () {
            viewerObj[fieldName] = new Viewer(document.getElementById(fieldName + 'FileBox'), {navbar: false});
        });
        addClick();
    }

    /**
     * 初始化文件上传
     * @param fieldName
     * @param maxCount
     */
    this.initUploadFile = function (fieldName, maxCount) {
        maxCount = maxCount ? parseInt(maxCount) : 3
        maxCount = maxCount < 0 ? 3 : maxCount;
        var id = '#' + fieldName;
        var fileBoxId = id + 'FileBox';
        var uploadBtnId = id + 'UploadBtn';
        var deleteBtnClass = '.delete-' + fieldName + '-btn';

        function init() {
            var files = $(id).val();
            var fileArray = files ? files.split(',') : [];

            var html = '';
            if (fileArray.length > 0) {
                html += [
                    '<div class="form-upload-file">',
                    '   <table class="layui-table" lay-size="sm">',
                    '       <colgroup>',
                    '           <col>',
                    '           <col width="60">',
                    '       </colgroup>',
                    '       <thead>',
                    '       <tr>',
                    '           <th>文件名</th>',
                    '           <th class="operation">操作</th>',
                    '       </tr>',
                    '       </thead>',
                    '       <tbody>',
                ].join('');
                for (var i = 0, file; file = fileArray[i++];) {
                    var idNameArray = file.split(':');
                    var fileId = idNameArray[0];
                    var fileName = idNameArray[1];
                    html += [
                        '<tr>',
                        '   <td>' + fileName + '</td>',
                        '   <td class="operation">',
                        '       <button type="button" class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-' + fieldName + '-btn" data-id="' + fileId + '" data-name="' + fileName + '" data-field-name="' + fieldName + '">删除</button>',
                        '   </td>',
                        '</tr>',
                    ].join('');
                }
                html += [
                    '       </tbody>',
                    '   </table>',
                    '</div>'
                ].join('');
            }

            $(fileBoxId).empty();
            $(fileBoxId).append(html);

            if (fileArray.length < maxCount) {
                $(uploadBtnId).removeClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', false);
            } else {
                $(uploadBtnId).addClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', true);
            }
        }

        function addClick() {
            upload.render({
                elem: uploadBtnId,
                url: 'api/file/v2/upload-file',
                accept: 'file',
                acceptMime: [
                    'application/pdf',
                    'application/msword',
                    'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
                    'application/vnd.ms-excel',
                    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
                    'application/vnd.ms-powerpoint',
                    'application/vnd.openxmlformats-officedocument.presentationml.presentation',
                    'application/vnd.ms-works',
                    'text/plain',
                    'application/x-rar',
                    'application/x-zip-compressed'
                ].join(''),
                field: 'file',
                exts: 'pdf|doc|docx|xls|xlsx|ppt|pptx|wps|txt|rar|zip',
                name: fieldName,
                done: function (res, index, upload) {
                    var name = this.name;
                    var files = $('#' + this.name).val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId + ':' + res.data.fileName;
                    $('#' + name).val(files);
                    init();
                },
                error: function (index, upload) {
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                    var percent = n + '%'
                    console.log(percent);
                    console.log(elem);
                    console.log(res);
                    console.log(index);
                    element.progress('demo-' + index, n + '%');
                }
            });
            $(document).on('click', deleteBtnClass, function () {
                var id = this.dataset.id;
                var name = this.dataset.name;
                var fieldName = this.dataset.fieldName;
                var files = $('#' + fieldName).val().replace(id + ':' + name, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) == ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) == ',') {
                    files = files.substring(0, files.length - 1);
                }
                $('#' + fieldName).val(files);
                init();
            });
        }

        init();
        addClick();
    }

    /**
     * 初始化视频上传
     * @param fileName
     * @param maxCount
     */
    this.initUploadVideo = function (fileName, maxCount) {
        maxCount = maxCount ? parseInt(maxCount) : 1
        maxCount = maxCount < 0 ? 1 : maxCount;
        var id = '#' + fileName;
        var fileBoxId = id + 'FileBox';
        var uploadBtnId = id + 'UploadBtn';
        var deleteBtnClass = '.delete-' + fileName + '-btn';

        function init() {
            var fileIds = $(id).val();
            var fileIdArray = fileIds ? fileIds.split(',') : [];
            var html = '';
            for (var i = 0, fileId; fileId = fileIdArray[i++];) {
                html += [
                    '<div class="form-upload-video">',
                    '   <video width="320" height="240" controls>',
                    '       <source src="route/file/download/true/' + fileId + '" type="video/mp4">',
                    '       您的浏览器不支持 video 标签',
                    '   </video>',
                    '   <a class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-' + fileName + '-btn" href="javascript:void(0);" data-id="' + fileId + '" data-name="' + fileName + '">',
                    '       <i class="fa fa-trash-o"></i>',
                    '   </a>',
                    '</div>',
                ].join('');
            }

            $(fileBoxId).empty();
            $(fileBoxId).append(html);

            if (fileIdArray.length < maxCount) {
                $(uploadBtnId).removeClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', false);
            } else {
                $(uploadBtnId).addClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', true);
            }
        }

        function addClick() {
            upload.render({
                elem: uploadBtnId,
                url: 'api/file/v2/upload-video',
                accept: 'video',
                acceptMime: 'video/mp4',
                exts: 'mp4',
                field: 'video',
                name: fileName,
                done: function (res, index, upload) {
                    var name = this.name;
                    var files = $('#' + this.name).val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId;
                    $('#' + name).val(files);
                    init();
                },
                error: function (index, upload) {
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                    var percent = n + '%'
                    console.log(percent);
                    console.log(elem);
                    console.log(res);
                    console.log(index);
                    element.progress('demo-' + index, n + '%');
                }
            });
            $(document).on('click', deleteBtnClass, function () {
                var name = this.dataset.name;
                var id = this.dataset.id;
                var files = $('#' + name).val().replace(id, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) == ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) == ',') {
                    files = files.substring(0, files.length - 1);
                }
                $('#' + name).val(files);
                init();
            });
        }

        init();
        addClick();
    }

    /**
     * 初始化音频上传
     * @param fileName
     * @param maxCount
     */
    this.initUploadAudio = function (fileName, maxCount) {
        maxCount = maxCount ? parseInt(maxCount) : 1
        maxCount = maxCount < 0 ? 1 : maxCount;
        var id = '#' + fileName;
        var fileBoxId = id + 'FileBox';
        var uploadBtnId = id + 'UploadBtn';
        var deleteBtnClass = '.delete-' + fileName + '-btn';

        function init() {
            var fileIds = $(id).val();
            var fileIdArray = fileIds ? fileIds.split(',') : [];
            var html = '';
            for (var i = 0, fileId; fileId = fileIdArray[i++];) {
                html += [
                    '<div class="form-upload-video">',
                    '   <audio src="route/file/download/true/' + fileId + '" controls>',
                    '       您的浏览器不支持 audio 标签。',
                    '   </audio>',
                    '   <a class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-' + fileName + '-btn" href="javascript:void(0);" data-id="' + fileId + '" data-name="' + fileName + '">',
                    '       <i class="fa fa-trash-o"></i>',
                    '   </a>',
                    '</div>',
                ].join('');
            }

            $(fileBoxId).empty();
            $(fileBoxId).append(html);

            if (fileIdArray.length < maxCount) {
                $(uploadBtnId).removeClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', false);
            } else {
                $(uploadBtnId).addClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', true);
            }
        }

        function addClick() {
            upload.render({
                elem: uploadBtnId,
                url: 'api/file/v2/upload-audio',
                accept: 'audio',
                acceptMime: [
                    'audio/wav',
                    'audio/mp3'
                ],
                exts: 'wav|mp3',
                field: 'audio',
                name: fileName,
                done: function (res, index, upload) {
                    var name = this.name;
                    var files = $('#' + this.name).val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId;
                    $('#' + name).val(files);
                    init();
                },
                error: function (index, upload) {
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                    var percent = n + '%'
                    console.log(percent);
                    console.log(elem);
                    console.log(res);
                    console.log(index);
                    element.progress('demo-' + index, n + '%');
                }
            });
            $(document).on('click', deleteBtnClass, function () {
                var name = this.dataset.name;
                var id = this.dataset.id;
                var files = $('#' + name).val().replace(id, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) == ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) == ',') {
                    files = files.substring(0, files.length - 1);
                }
                $('#' + name).val(files);
                init();
            });
        }

        init();
        addClick();
    }

    /**
     * 初始化日期
     * @param opt
     */
    this.initDate = function (opt) {
        laydate.render({
            elem: '#' + opt.id,
            type: opt.datetype,
            format: opt.dateformat,
            value: opt.dateDefaultValue,
            min: opt.dataMinValue,
            max: opt.dataMaxValue,
            trigger: 'click'
        });
    }

    /**
     * 初始化checkbox数据
     * @param form
     * @param checkedData
     * @param id
     */
    this.initCheckboxData = function (form, formName, checkedData, id) {
        var dataArray = checkedData.split(',');
        var obj = {};
        for (var i = 0, data; data = dataArray[i++];) {
            obj[id + '[' + data + ']'] = true;
        }
        form.val(formName, obj);
    }

    /**
     * 设置复选框值
     * @param formData
     * @param id
     */
    this.setCheckboxValue = function (formData, id) {
        formData.field[id] = restAjax.checkBoxToString(formData.field, id);
    }

    /**
     * 设置开关值
     * @param formData
     * @param id
     */
    this.setSwitchValue = function (formData, id) {
        formData.field[id] = formData.field[id] ? 1 : 0;
    }

}